@@ -6,8 +6,9 @@ test_that("phylospatial function works on example data", {
       expect_no_error(phylospatial(ps_get_comm(ps), ps$tree))
 
       # test taxonomic reconciliation
-      expect_warning(phylospatial(ps_get_comm(ps)[, -3],
-                                   ape::drop.tip(ps$tree, 1)))
+      expect_warning(x <- phylospatial(ps_get_comm(ps)[, -3],
+                                   ape::drop.tip(ps$tree, 1)),
+                     "Dropping")
 })
 
 test_that("taxa are not scrambled during data transformations", {
