@@ -14,14 +14,15 @@ test_that("`ps_canape()` runs without error on simulated data", {
             fun = "nullmodel", method = "curveball",
             n_rand = 3, burnin = 100, progress = FALSE)))
 
-      expect_no_error(ps_canape(ps_rand(
-            ps_simulate(data_type = "prob"),
-            fun = "quantize", method = "curveball",
-            n_rand = 3, progress = FALSE)))
-
       expect_no_error(ps_canape(ps_rand(
             ps_simulate(data_type = "binary", spatial_type = "none"),
             fun = "nullmodel", method = "curveball",
             n_rand = 3, burnin = 100, progress = FALSE)))
 
+      if(requireNamespace("nullcat")){
+            expect_no_error(ps_canape(ps_rand(
+                  ps_simulate(data_type = "prob"),
+                  fun = "quantize", method = "curvecat",
+                  n_rand = 3, progress = FALSE)))
+      }
 })
